/*******************************************************************************
 *                                                                             *
 *                          Written by Jack Crotty                             *
 *                           Tullahoma, Tennessee                              *
 *                              Copyright 2001                                 *
 *                            All Rights Reserved                              *
 *                           Updated 29 April 2001                             *
 *                            Jack___46@Yahoo.com                              *
 *                                 Monitor2.c                                  *
 *                                                                             *
 *******************************************************************************/
#include <hc12.h>
#include <stdio.h>

#define ABOVE_FLASH     0xC000      /* Used to compare Address for Flash loads */
#define BLINKRATE       2           /* LED Blink Rate for S-Record operations  */
#define BS              8           /* ASCII backspace                         */
#define CHARSPERROW     16          /* Number of characters per line (Dump cmd)*/
#define COMPLETED       3           /* Signals process is complete             */
#define CR              0xD         /* ASCII Carrage return                    */
#define DECREMENT       0x12        /* Flag to indicate user wants to dec addr */
#define ESCAPE          0x1B        /* ASCII for escape key (terminates cmds)  */
#define FALSE           0           /* Boolean indication for NOT True         */
#define FLASH           0x8000      /* Start of Flash memory in system         */
#define INCREMENT       0x11        /* Flag to indicate user wants to inc addr */
#define LAST_PPAGE      0x1F        /* Last Program Flash page (set 128K = 07) *
                                     * redefine and compile, or change value   *
                                     * in RAM location (Last_PPAGE--adr in LST */
#define LED             0x40        /* Bit definition for LED on Technological *
                                     *          Arts DX 812 board              */
#define LF              0xA         /* ASCII Line feed                         */
#define PERIOD          '.'         /* ASCII for period (used as delimiter)    */
#define RE              0x20        /* Receiver ready bit in SCI register      */
#define ROWSPERSCREEN   24          /* Number of rows per screen dump          */
#define SPACE           0x20        /* ASCII space                             */
#define SUCCESS         2           /* Signals success for process             */
#define TRANSCOMPLETE   0x40        /* Transmit complete SCI Status register.  */
#define TE              0x80        /* Transmitter ready bit in SCI register   */
#define TERMINATED      0x10        /* Process terminated with Escape or CR    */
#define TRUE            1           /* Flag to value to indicate success truth */
#define UPPER_REG       0x1FF       /* Last memory address for registers       *

 ***************************************************************************
 *                                                                         *
 ************************* Global variables ********************************
 *                                                                         *
 ***************************************************************************/

unsigned char  *Address,    /* Holds the address of character entered for  *
                             * S-Record, memory examine, and fill commands */
                Adrhi,      /* Holds upper half for the above address      */
                Adrlo;      /* Holds lower half for the above address      */
char            Banking,    /* When True Records contain and set PPAGE     */
                CheckSum,   /* CheckSum for S-Record operations            */
                Count,      /* Holds record count for S-Record routines    */
                Curpage,    /* Contains Program page for Flash operations  */
                Data,       /* Data being read and passed to Flash program */
                Error,      /* Error indication for operation              */
                Last_PPAGE, /* Last_PPAGE contains the last allowed page   */
                MSBadr;     /* Most significant nibble for Flash address   */

/***************************************************************************
 *                                                                         *
 ************************** Function Prototypes ****************************
 *                                                                         *
 ***************************************************************************/

  void Back             (char);
  char BuildAddress     (char);
  char BuildChar        (char);
  void Clr_Screen       (void);
  void CRLF             (void);
  void Dump             (void);
  void DisplayCommands  (void);
  void Display_New_Addr (char);
  void Display_New_Char (char);
  void EraseFlash       (void);
  void Examine          (void);
  void Execute          (void);
  void Fill             (void);
  void GetAddress       (void);
  char GetByte          (void);
  char GetChar          (void);
  char GetSRecord       (void);
  void Initialize       (void);
  void LED_Off          (void);
  void LED_On           (void);
  void Load_SRecord     (void);
  void PutByte          (unsigned char);
  void PutChar          (char);
  void PutString        (char *);
  void PutWord          (unsigned int);
  void Process_Cmd      (void);
  void Program_Flash    (void);
  void Reset            (void);
  void Verify_SRecord   (void);
  void WriteFlash       (void);

/***************************************************************************
 *                                                                         *
 ***************************** Start Main Program **************************
 *                                                                         *
 ***************************************************************************/

main(void)
{
    /* Basic Monitor Program for Technological Arts *
     * Motorola 68HC812A4 based processor system.   *
     * Program resides in EEPROM, and is, therefore *
     * limited to 4K (including Vectors and Loader) */

    Initialize();               /* Initialize System state  */
    Process_Cmd();			    /* Process user commands    *
                                 * until terminated by user */
    return(0);
}

/***************************************************************************
 *                                                                         *
 ***************************** End Main Program ****************************
 *                                                                         *
 ***************************************************************************/

/*******************************************
 *                                         *
 * Function to Load S-Records into memory. *
 * Uses GetSRecord                         *
 *                                         *
 *******************************************/
void Load_SRecord (void)
{
    char temp;

    Error = FALSE;

    while (( GetSRecord()) != COMPLETED )
    {
        /************************************************
         *                                              *
         * GetSRecord sets Address and Count (also      *
         * updates CheckSum) for this record, so        *
         * simply load the record based on these values *
         *                                              *
         ************************************************/
        while (Count > 1)
        {
            /* Store the data into memory locations     */
            temp       = GetByte();
            *Address++ = temp;
            CheckSum  += temp;
            Count--;
        }
        /***********************************************
         *                                             *
         * Checksum for this record remains, get it,   *
         * and compare it to the value calculated.     *
         *                                             *
         ***********************************************/

        if (GetByte() != ~CheckSum)
            /* Error occurred in loading this record   */
            Error = TRUE;
    }/* Load has completed */
}

/*********************************************
 *                                           *
 * Function to Program S-Records into Flash. *
 * Uses GetSRecord                           *
 *                                           *
 *********************************************/
void Program_Flash (void)
{
    char done = FALSE;

    Error = FALSE;

    while (GetSRecord() != COMPLETED )
    {
        /************************************************
         *                                              *
         * GetSRecord sets Address and Count (also      *
         * updates CheckSum) for this record, so        *
         * simply load the record based on these values *
         *                                              *
         ************************************************/
        if (Address >= (unsigned char *)ABOVE_FLASH)
        {
            /* S1 Record is attempting to write to Last Flash page.  *
             * S2 would set PPAGE directly and leave address between *
             * 8000 and BFFF.  So limit address to 8000 - BFFF, and  *
             * set LAST_PPAGE (512 K = 1F, 128 K would be 7).        */
            Address = (unsigned char *)((int)Address & (ABOVE_FLASH-1));
            PPAGE = Last_PPAGE;     /* Set PPAGE (Last_PPAGE is RAM */
        }
        while (Count > 1)
        {
            /* Get data into memory locations     */
            Data = GetByte();
            if (Address >= (unsigned char *)FLASH)
                WriteFlash();
            else
                *Address = Data;

            Address++;
            CheckSum  += Data;
            Count--;
        }
        /***********************************************
         *                                             *
         * Checksum for this record remains, get it,   *
         * and compare it to the value calculated.     *
         *                                             *
         ***********************************************/
        if (GetByte() != ~CheckSum)
            /* Error occurred in loading this record   */
            Error = TRUE;
    }/* Load has completed */
}

/*******************************************
 *                                         *
 * Function to Verify S-Records to Memory. *
 * Uses GetSRecord                         *
 *                                         *
 *******************************************/
void Verify_SRecord (void)
{
    char temp;

    Error  = FALSE;
    PORTT &= ~LED;

    while (GetSRecord() != COMPLETED )
    {
        /************************************************
         *                                              *
         * GetSRecord sets Address and Count (also      *
         * updates CheckSum) for this record, so simply *
         * compare the record to values in memory.      *
         *                                              *
         ************************************************/
        while (Count > 1)
        {
            temp = GetByte();

            if ((unsigned int) Address <= UPPER_REG)

               /* Write value to register */
               *Address = temp;
            else
                /* Not a register address, so compare memory */
                if(*Address != temp)
                    Error = TRUE;

            Address++;
            CheckSum  += temp;
            Count--;
        }
        /***********************************************
         *                                             *
         * Checksum for this record remains, get it,   *
         * and ignore it (memory contents matched.     *
         *                                             *
         ***********************************************/
        temp = GetByte();
    }/* Verify is complete */
}

/*********************************************************************
 *                                                                   *
 * Function:                                                         *
 * GetSRecord(void)                                                  *
 *                                                                   *
 * Receives Motorola 'S' formated memory load files.                 *
 * S0 record is file information, and are ignored by this function   *
 * S1 record has address followed by the byte count for this record  *
 * S2 record has 24 bit address (compared to 16 for S1), byte count  *
 * S9 record indicates the file is terminating                       *
 *                                                                   *
 * if S1 is received                                                 *
 * Function returns 'SUCCESS',                                       *
 * sets Address, sets Count, and                                     *
 * calculates CheckSum for Count and Address                         *
 *                                                                   *
 * if S2 is received                                                 *
 * Function sets PPAGE, returns 'SUCCESS',                           *
 * sets Address, sets Count, and                                     *
 * calculates CheckSum for Count and Address                         *
 *                                                                   *
 * if S9 is received                                                 *
 * Function returns 'COMPLETE'                                       *
 * after getting count and waiting for the number of characters      *
 *                                                                   *
 * uses GetChar(), GetByte(), GetAddress()                           *
 *********************************************************************/
char GetSRecord (void)
{
    static char count    = 0;
           char inchar,
                received = FALSE;

    while (!received)
    {
        /* Wait for 'S' (UPPERCASE) to be received. */
        while (GetChar() != 'S');
        /******************************************
         *                                        *
         * 'S' received, Get input, and ignore if *
         * it is not a 1,2 or 9 (S1-S2-S9 record) *
         *                                        *
         ******************************************/
        inchar = GetChar();

        if (++count > BLINKRATE)
        {
            PORTT = PORTT ^ LED;
            count = 0;
        }

        switch (inchar)
        {
            case '1':
                /************************************
                 *                                  *
                 * Get the Count, Address, and      *
                 * indicate Success for this record *
                 *                                  *
                 ************************************/
                Count    = GetByte();
                CheckSum = Count;

                /*******************************
                 *                             *
                 * Update Address and CheckSum *
                 *                             *
                 *******************************/
                GetAddress();

                if (!Banking)
                {
                    /* Non banking mode need to check if *
                     * operation is in second last 16K.  */
                    if ((unsigned int) Address <= (ABOVE_FLASH-1) &&
                       ((unsigned int) Address >=  FLASH))
                        /* record writing to second last 16 K */
                        PPAGE = (Last_PPAGE - 1);
                    else
                        if ((unsigned int) Address >= ABOVE_FLASH)
                            /* record is writing to last 16 K */
                            PPAGE = (Last_PPAGE);
                }
                Count    = Count - 2;
                received = SUCCESS;
                break;

            case '2':
                /**************************************
                 *                                    *
                 * Get the Count, 24 bit Address, and *
                 * indicate Success for this record.  *
                 *                                    *
                 **************************************/
                Count    = GetByte();
                CheckSum = Count;

                /*******************************
                 *                             *
                 * Update Address and CheckSum *
                 *                             *
                 *******************************/
                MSBadr    = GetByte();
                CheckSum += MSBadr;

                GetAddress();
                PPAGE = (MSBadr<<2 + Adrhi>>6);

                if (PPAGE > Last_PPAGE)
                    Error = TRUE;

	            Address = (unsigned char *)((int)Address & (ABOVE_FLASH-1));


                Count    = Count - 3;
                received = SUCCESS;
                break;

            case '9':
                /********************************************
                 *                                          *
                 * End of record has been received. Need to *
                 * get count, and the remaining characters. *
                 * ie S9030000FC (checksum for S9 record is *
                 * ignored--if there are no errors to this  *
                 * point, information loaded correctly.     *
                 *                                          *
                 ********************************************/

                Count = GetByte();

                while (Count)
                {
                    inchar = GetByte();
                    Count--;
                }
                received = COMPLETED;
                inchar   = GetChar();   /* Imagecraft Compiler, and Motorola Assembler *
                                         * both send a carriage return after the end   *
                                         * of the Record -- May need to be removed for *
                                         * records generated from other sources....... *
                                         * if removed it will result in a double enter *
                                         * command line from monitor would lockup if   *
                                         * program does not send terminating character */
                break;
        } /* end switch */
    } /* end while (!done) */

    /************************************************************
     *                                                          *
     * When program comes here, while has completed, therefore, *
     * Record was either S1 or S9 so return indication          *
     *                                                          *
     ************************************************************/
    return (received);
}

/****************************************************
 *                                                  *
 * Function to Get 2 ASCII characters from SCI port *
 * using GetChar.  Assumes characters are 0 - 9 or  *
 * A - F. Returns characters combined as a byte     *
 *                                                  *
 ****************************************************/
char GetByte (void)
{
    char temphi,
         templo;

    temphi = GetChar();

    if(temphi > '9')
    /* Character is an alpha *
     * correct the nibble.   */
        temphi += 9;        /* A would be 41 hex (1 when it's upper nibble */

    temphi = temphi << 4;   /* Make input character the upper nibble. */
    templo = GetChar();     /* Get least significant nibble adjust    *
                             * it's value, and strip off upper nibble */

    if ( templo > '9')
        templo += 9;

    templo &= 0x0F;

    /* Assemble nibbles, and return *
     * the 'new' byte to the caller */
    return (temphi |= templo);
}

/********************************************************
 *                                                      *
 * Function to Get 4 ASCII characters from SCI port     *
 * using GetByte.  Places entered nibbles of address    *
 * in Global characters Adrhi and Adrlo before placing  *
 * them into Global character pointer (Address). Also   *
 * updates the checksum for received address lo and hi. *
 *                                                      *
 ********************************************************/
void GetAddress(void)
{
    Adrhi     = GetByte();
    Adrlo     = GetByte();
    CheckSum += (Adrhi) | (Adrlo);
    Address   = (unsigned char *) (Adrhi * 256 | Adrlo );
}

/********************************************************
 *                                                      *
 * Function to Build address from input on SCI port     *
 * using GetChar.  Address is built a nibble at a time  *
 * input is terminated with a space or enter.  Address  *
 * initially is set to zero.  Function uses Global      *
 * characters Adrhi and Adrlo for build and places the  *
 * final result in Global pointer (Address).            *
 *                                                      *
 ********************************************************/
char BuildAddress(char message)
{
    char done = FALSE;
    char input;

    if(message)
    {
        CRLF();
        PutString (" Enter Address:  ");
    }
    /* Make default what was left in address *
     * variables (Adrhi and Adrlo )...       */
    PutByte(Adrhi);
    PutByte(Adrlo);

    while(!done)
    {
        /* Get next character from user */
        input = GetChar();
        if ((input >= '0')  &&
            (input <= '9'))
        {
            /* User entered a digit */
            input &= 0xF;
            Display_New_Addr(input);
        }
        else
            if ((input == SPACE) ||
                (input == CR))
                /* User completed input */
                done = TRUE;
            else
            {
                if ((input >= 'A')  &&
                    (input <= 'F')
                           ||
                    (input >= 'a')  &&
                    (input <= 'f'))
                {
                    /* Correct hexidecimal digit */
                    input = (input & 0x7) + 9;
                    Display_New_Addr(input);
                }
                else
                    if ((input == ESCAPE) ||
                        (input == PERIOD))
                        /* User terminated input */
                        return (TERMINATED);
            }
    }
    Address = (unsigned char *) (Adrhi * 256 + Adrlo );
    return (COMPLETED);
}

/**********************************************************
 *                                                        *
 * Function to Build character from input on SCI port     *
 * using GetChar.  Character is built a nibble at a time  *
 * input is terminated with a space.  Character initially *
 * is set to zero.  Function uses Global character Adrlo  *
 * and places the final result in Global variable Adrlo.  *
 *                                                        *
 **********************************************************/
char BuildChar(char value)
{
    char done = FALSE;
    char input;

    PutByte(value);     /* Display default character on screen */

    while(!done)
    {
        /* Get next character from user */
        input = GetChar();
        if ((input >= '0') &&
            (input <= '9'))
        {
            /* User entered a digit */
            value = (value * 16) | (input & 0x0F);
            Display_New_Char(value);
        }
        else
            if ((input == SPACE) ||
                (input == CR))
                /* User completed input */
                done = TRUE;
            else
            {
                if ((input >= 'A')  &&
                    (input <= 'F')
                           ||
                    (input >= 'a')  &&
                    (input <= 'f'))
                {
                    /* Correct hexidecimal digit */
                    value = ((value * 16) | ((input & 0x7) + 9));
                    Display_New_Char(value);
                }
                else
                    if ((input == ESCAPE) ||
                        (input == PERIOD))
                        /* User terminated input */
                        return (TERMINATED);
                    else
                        if (input == '+')
                            return (INCREMENT);
                        else
                            if(input == '-')
                                return (DECREMENT);
            }
    }
    Adrlo = value;
    return (COMPLETED);
}

/**********************************************************
 *                                                        *
 * Function to Examine memory contents for an address and *
 * then allow the user to accept the contents, change the *
 * contents, or go forward or backward in memory.         *
 *                                                        *
 **********************************************************/
void Examine(void)
{
    char status;

    PutString("Examine Memory");

    if (BuildAddress(TRUE) != TERMINATED)
    {
        PutString (" ");
        while (status != TERMINATED)
        {
            status = BuildChar (*Address);

            if (status == COMPLETED)
                *Address++ = Adrlo;
            else
                if (status == INCREMENT)
                    Address++;
                else
                    if (status == DECREMENT)
                        Address--;

            if (status != TERMINATED)
            {
                CRLF();
                PutString (" Enter Address:  ");
                PutWord((int)Address);
                PutString(" ");
            }
        }
    }
}
/**********************************************************
 *                                                        *
 * Function to Execute a program from address entered     *
 *                                                        *
 **********************************************************/

void Execute (void)
{
    PutString(" Run Code");
    if ( BuildAddress(TRUE) != TERMINATED )
    {
        /* User entered valid address */
        asm ("LDY _Address");
        asm ("JMP 0, Y");
    }
}

/**********************************************************
 *                                                        *
 * Function to Fill memory from a Start address to an End *
 * address with a character. The addresses and character  *
 * are input by the user on the SCI port                  *
 *                                                        *
 **********************************************************/
void Fill(void)
{
    unsigned char *start,
                  *stop;
    int  temp;

    PutString("Fill Memory");

    if ( BuildAddress(TRUE) != TERMINATED )
    {
        /* User entered Start address */
        start = Address;

        if ( BuildAddress(TRUE) != TERMINATED )
        {
            /* User Entered End address */
            stop = Address;
            CRLF();
            PutString(" Enter Char:     ");
            if (BuildChar(0) != TERMINATED)
            {
                /* User entered Fill character */
                if (start <= stop)
                    /* Addresses are in order fill memory */
                    while (start <= stop)
                    {
                        *start = Adrlo;
                        if (*start != Adrlo)
                        {
                            /* Value did NOT store properly */
                            CRLF();
                            PutString(" ERROR at: ");
                            temp = (int)start;
                            PutWord(temp);
                            CRLF();
                            start = stop;
                        }
                        start++;
                    }
                else
                    /* Address out of range */
                    puts("\n End < Start aborted!\n\n");
            }
        }
    }
}

/**********************************************************
 *                                                        *
 * Function to Dump memory contents to screen             *
 *                                                        *
 **********************************************************/
void Dump (void)
{
    unsigned char *adr_ptr,
                  *data_ptr,
                   done = FALSE,
                   i,
                   input,
                   j;
    int  dump_adr;

    PutString("Dump Memory");

    if (BuildAddress(TRUE) != TERMINATED)
    {
        adr_ptr = Address;
        dump_adr = Adrhi * 256 + Adrlo;
        CRLF();
        while ( !done)
        {
            for( i = 0; i < ROWSPERSCREEN; i++)
            {
                /* Start by outputing address to screen */
                PutString (" ");
                PutWord(dump_adr);
                dump_adr += CHARSPERROW;
                /* Adjust pointer for data byte output */
                data_ptr = adr_ptr;
                for( j = 0; j < CHARSPERROW; j++)
                {
                    PutString(" ");
                    PutByte(*data_ptr++);
                }
                /* Row of data bytes have been output *
                 * a space then the ASCII from memory */
                PutString(" ");

                for( j = 0; j < CHARSPERROW; j++)
                {
                    if ((*adr_ptr >= SPACE) &&
                        (*adr_ptr <= '~'))
                        PutChar(*adr_ptr);
                    else
                        PutChar('.');
                    adr_ptr++;
                }
                CRLF();
            }
            CRLF();
            PutString(" Period/Esc to end.");
            input = GetChar();
            if ((input == ESCAPE) ||
                (input == PERIOD))
                done = TRUE;
            else
                puts("\n");

        }
    }
}

/********************************************************
 *                                                      *
 * Function to add nibble passed to routine as the Least*
 * significant nibble of address being built            *
 *                                                      *
 ********************************************************/
void Display_New_Addr(char in)
{
    Adrhi = (unsigned char)((Adrhi*16) + (Adrlo/16));
    Adrlo = (unsigned char)((Adrlo*16) + in);
    Back(4);
    PutByte(Adrhi);
    PutByte(Adrlo);
}

/*********************************************************
 *                                                       *
 * Function to add nibble passed to routine as the Least *
 * significant nibble of character being built           *
 *                                                       *
 *********************************************************/
void Display_New_Char(char in)
{
    Back(2);
    PutByte(in);
}

/****************************************************
 *                                                  *
 * Function to Get an ASCII character from SCI port *
 * Returns received character to calling routine.   *
 *                                                  *
 ****************************************************/
char GetChar (void)
{
    while (!(SC0SR1 & RE));   /* Wait for character */
    return (SC0DRL);
}

/*********************************************************
 *                                                       *
 * Function to output ASCII character passed on SCI Port *
 *                                                       *
 *********************************************************/
void PutChar (char outchar)
{
    while (!(SC0SR1 & TRANSCOMPLETE));
    SC0DRL = outchar;
}

void PutString (char *in)
{
    while (*in)
        PutChar (*in++);
}

/*********************************************************
 *                                                       *
 * Function to output Carrage return Linefeed to SCI Port*
 *                                                       *
 *********************************************************/
void CRLF(void)
{
    PutChar (CR);
    PutChar (LF);
}

/************************************************
 *                                              *
 * Function to erase passed count of characters *
 * and leave the cursor at the last character   *
 * that it removes.                             *
 *                                              *
 ************************************************/
void Back (char count)
{
    while (count--)
    {
        PutChar(BS);
        PutChar(SPACE);
        PutChar(BS);
    }
}

/*******************************************************
 *                                                     *
 *  Function to output Word passed as 4 Hex digits     *
 *                                                     *
 *******************************************************/
void PutWord (unsigned int outword)
{
    unsigned char temp;
    temp = (outword/256);
    PutByte(temp);
    temp = outword & 0xFF;
    PutByte(temp);
}

/*******************************************************
 *                                                     *
 * Function to output Character passed as 2 Hex digits *
 *                                                     *
 *******************************************************/
void PutByte (unsigned char outchar)
{
    unsigned char temp;      /* Used for input while manipulating */

    temp = outchar/16;
    if (temp > 9)
        /* Character is an alpha */
        temp = temp + 0x37;
    else
        temp = temp + 0x30;

    PutChar (temp);         /* Output most significant nibble */

    temp = outchar &0xF;    /* Strip MS nibble */
    if (temp > 9)
        temp += 0x37;
    else
        temp += 0x30;

    PutChar (temp);  /* Output least significant nibble */
}

/*******************************************************
 *                                                     *
 * Function to Display Commands defined for monitor    *
 *                                                     *
 *******************************************************/
void DisplayCommands(void)
{
    puts ("\n\n Copyright Jack Crotty all Rights Reserved");
    puts (" Commands are:\n");
    puts (" d = Dump");
    puts (" e = Examine");
    puts (" E = Erase Flash");
    puts (" f = Fill");
    puts (" g = Run Code");
    puts (" l = Load RAM");
    puts (" p = Load Flash");
    puts (" t = Flip LED");
    puts (" v = Verify Load");
    puts (" ? = Cmds");
}

/*******************************************************
 *                                                     *
 * Function to Initialize the state of the processor   *
 *                                                     *
 *******************************************************/
void Initialize(void)
{
    char    temp;

    PEAR    = 4;        /* Enable read/write signal for external RAM     */
    MXAR    = 7;        /* Enable Addr 16,17, and 18                     */
    CSCTL0  = 0x30;     /* Enable CSD & CSPO                             */
    CSCTL1  = 0x10;     /* Enable lower half of memory map               */
    MODE    = 0xB0;     /* Set for Normal Expanded Narrow (176 = B0 Hex) */
    WINDEF  = 0xC0;     /* Enable data and program windows               */

    COPCTL  = 0;        /* Turn off COP                                  */
    setbaud(BAUD38K);   /* Set baud rate to 38400                        */

    while ( SC0SR1 & 0x20 )
        /* Read register to clear flag RDRF */
        temp = SC0DRL;

    Clr_Screen();
    Last_PPAGE = LAST_PPAGE;/* Last PPAGE defaults to 512k can be set by *
                             * changing RAM using monitor or recompiling */
    Banking    = FALSE;     /* S Records will write directly to 8000 - FFFF */
    DDRT       = 208;       /* Make Port T B7 B6 B4 Output */
}

/********************************
 *                              *
 * Function to Clear the screen *
 *                              *
 ********************************/

void Clr_Screen (void)
{
    PutChar (ESCAPE);
    PutString ("[2J");
}

/************************************************
 *                                              *
 * Function to Process Commands entered by user *
 *                                              *
 ************************************************/
void Process_Cmd (void)
{
	char inchar;

    DisplayCommands();
    do
	{
        CRLF();
        PutString(" Enter Command: ");
        inchar = GetChar();

        switch (inchar)
		{
            case 'd':
                Dump();
                break;

            case 'e':
                Examine();
                break;

            case 'E':
                PutString ("Erase Flash Continue?");
                inchar = GetChar();
                Back(9);               /* Erase continue from screen */
                if ((inchar == 'y') ||
                    (inchar == 'Y'))
                {
                    EraseFlash();
                    if ( Error)
                        PutString("FAILED");
                    else
                        PutString("Passed");
                }
                else
                    PutString ("Aborted");
                break;

            case 'f':
                Fill();
                break;

            case 'g':
                Execute();
                break;

            case 'l':
                PutString("Load RAM");
                Error = FALSE;
                Load_SRecord();
                LED_Off();
                break;

           case 'p':
                PutString("Prog Flash");
                Program_Flash();
                CRLF();
                LED_Off();
                break;

            case 't':
                PutString("LED Toggled");
                PORTT = PORTT ^ LED;
                break;

            case 'v':
                PutString("Verify Load");
                Verify_SRecord();
                if (Error)
                {
                    PutString(" FAILED");
                    LED_On();
                }
                else
                {
                    PutString(" OK");
                    LED_Off();
                }
                break;

            case '?':
                Clr_Screen();
                DisplayCommands();
                break;
		}// end switch
    } while (inchar != 02);
}
